@@ -2,7 +2,7 @@
 // detail/handler_tracking.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -99,8 +99,8 @@ public:
 
   private:
     // Disallow copying and assignment.
-    location(const location&) ASIO_DELETED;
-    location& operator=(const location&) ASIO_DELETED;
+    location(const location&) = delete;
+    location& operator=(const location&) = delete;
 
     friend class handler_tracking;
     const char* file_;
