@@ -2,7 +2,7 @@
 // experimental/basic_channel.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -111,7 +111,7 @@ private:
   template <typename... PayloadSignatures,
       ASIO_COMPLETION_TOKEN_FOR(PayloadSignatures...) CompletionToken>
   auto do_async_receive(detail::channel_payload<PayloadSignatures...>*,
-      ASIO_MOVE_ARG(CompletionToken) token)
+      CompletionToken&& token)
     -> decltype(
         async_initiate<CompletionToken, PayloadSignatures...>(
           declval<initiate_async_receive>(), token))
@@ -167,10 +167,10 @@ public:
    */
   template <typename ExecutionContext>
   basic_channel(ExecutionContext& context, std::size_t max_buffer_size = 0,
-      typename constraint<
+      constraint_t<
         is_convertible<ExecutionContext&, execution_context&>::value,
         defaulted_constraint
-      >::type = defaulted_constraint())
+      > = defaulted_constraint())
     : service_(&asio::use_service<service_type>(context)),
       impl_(),
       executor_(context.get_executor())
@@ -178,7 +178,6 @@ public:
     service_->construct(impl_, max_buffer_size);
   }
 
-#if defined(ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
   /// Move-construct a basic_channel from another.
   /**
    * This constructor moves a channel from one object to another.
@@ -236,9 +235,9 @@ public:
   template <typename Executor1>
   basic_channel(
       basic_channel<Executor1, Traits, Signatures...>&& other,
-      typename constraint<
+      constraint_t<
           is_convertible<Executor1, Executor>::value
-      >::type = 0)
+      > = 0)
     : service_(other.service_),
       executor_(other.executor_)
   {
@@ -259,10 +258,10 @@ public:
    * constructor.
    */
   template <typename Executor1>
-  typename constraint<
+  constraint_t<
     is_convertible<Executor1, Executor>::value,
     basic_channel&
-  >::type operator=(basic_channel<Executor1, Traits, Signatures...>&& other)
+  > operator=(basic_channel<Executor1, Traits, Signatures...>&& other)
   {
     if (this != &other)
     {
@@ -273,7 +272,6 @@ public:
     }
     return *this;
   }
-#endif // defined(ASIO_HAS_MOVE) || defined(GENERATING_DOCUMENTATION)
 
   /// Destructor.
   ~basic_channel()
@@ -282,19 +280,19 @@ public:
   }
 
   /// Get the executor associated with the object.
-  const executor_type& get_executor() ASIO_NOEXCEPT
+  const executor_type& get_executor() noexcept
   {
     return executor_;
   }
 
   /// Get the capacity of the channel's buffer.
-  std::size_t capacity() ASIO_NOEXCEPT
+  std::size_t capacity() noexcept
   {
     return service_->capacity(impl_);
   }
 
   /// Determine whether the channel is open.
-  bool is_open() const ASIO_NOEXCEPT
+  bool is_open() const noexcept
   {
     return service_->is_open(impl_);
   }
@@ -323,7 +321,7 @@ public:
   }
 
   /// Determine whether a message can be received without blocking.
-  bool ready() const ASIO_NOEXCEPT
+  bool ready() const noexcept
   {
     return service_->ready(impl_);
   }
@@ -337,14 +335,38 @@ public:
    * @returns @c true on success, @c false on failure.
    */
   template <typename... Args>
-  bool try_send(ASIO_MOVE_ARG(Args)... args);
+  bool try_send(Args&&... args);
+
+  /// Try to send a message without blocking, using dispatch semantics to call
+  /// the receive operation's completion handler.
+  /**
+   * Fails if the buffer is full and there are no waiting receive operations.
+   *
+   * The receive operation's completion handler may be called from inside this
+   * function.
+   *
+   * @returns @c true on success, @c false on failure.
+   */
+  template <typename... Args>
+  bool try_send_via_dispatch(Args&&... args);
 
   /// Try to send a number of messages without blocking.
   /**
    * @returns The number of messages that were sent.
    */
   template <typename... Args>
-  std::size_t try_send_n(std::size_t count, ASIO_MOVE_ARG(Args)... args);
+  std::size_t try_send_n(std::size_t count, Args&&... args);
+
+  /// Try to send a number of messages without blocking, using dispatch
+  /// semantics to call the receive operations' completion handlers.
+  /**
+   * The receive operations' completion handlers may be called from inside this
+   * function.
+   *
+   * @returns The number of messages that were sent.
+   */
+  template <typename... Args>
+  std::size_t try_send_n_via_dispatch(std::size_t count, Args&&... args);
 
   /// Asynchronously send a message.
   /**
@@ -354,8 +376,8 @@ public:
   template <typename... Args,
       ASIO_COMPLETION_TOKEN_FOR(void (asio::error_code))
         CompletionToken ASIO_DEFAULT_COMPLETION_TOKEN_TYPE(executor_type)>
-  auto async_send(ASIO_MOVE_ARG(Args)... args,
-      ASIO_MOVE_ARG(CompletionToken) token);
+  auto async_send(Args&&... args,
+      CompletionToken&& token);
 
 #endif // defined(GENERATING_DOCUMENTATION)
 
@@ -366,9 +388,9 @@ public:
    * @returns @c true on success, @c false on failure.
    */
   template <typename Handler>
-  bool try_receive(ASIO_MOVE_ARG(Handler) handler)
+  bool try_receive(Handler&& handler)
   {
-    return service_->try_receive(impl_, ASIO_MOVE_CAST(Handler)(handler));
+    return service_->try_receive(impl_, static_cast<Handler&&>(handler));
   }
 
   /// Asynchronously receive a message.
@@ -380,22 +402,22 @@ public:
   template <typename CompletionToken
       ASIO_DEFAULT_COMPLETION_TOKEN_TYPE(executor_type)>
   auto async_receive(
-      ASIO_MOVE_ARG(CompletionToken) token
+      CompletionToken&& token
         ASIO_DEFAULT_COMPLETION_TOKEN(Executor))
 #if !defined(GENERATING_DOCUMENTATION)
     -> decltype(
         this->do_async_receive(static_cast<payload_type*>(0),
-          ASIO_MOVE_CAST(CompletionToken)(token)))
+          static_cast<CompletionToken&&>(token)))
 #endif // !defined(GENERATING_DOCUMENTATION)
   {
     return this->do_async_receive(static_cast<payload_type*>(0),
-        ASIO_MOVE_CAST(CompletionToken)(token));
+        static_cast<CompletionToken&&>(token));
   }
 
 private:
   // Disallow copying and assignment.
-  basic_channel(const basic_channel&) ASIO_DELETED;
-  basic_channel& operator=(const basic_channel&) ASIO_DELETED;
+  basic_channel(const basic_channel&) = delete;
+  basic_channel& operator=(const basic_channel&) = delete;
 
   template <typename, typename, typename...>
   friend class detail::channel_send_functions;
@@ -403,7 +425,7 @@ private:
   // Helper function to get an executor's context.
   template <typename T>
   static execution_context& get_context(const T& t,
-      typename enable_if<execution::is_executor<T>::value>::type* = 0)
+      enable_if_t<execution::is_executor<T>::value>* = 0)
   {
     return asio::query(t, execution::context);
   }
@@ -411,7 +433,7 @@ private:
   // Helper function to get an executor's context.
   template <typename T>
   static execution_context& get_context(const T& t,
-      typename enable_if<!execution::is_executor<T>::value>::type* = 0)
+      enable_if_t<!execution::is_executor<T>::value>* = 0)
   {
     return t.context();
   }
@@ -426,18 +448,18 @@ private:
     {
     }
 
-    const executor_type& get_executor() const ASIO_NOEXCEPT
+    const executor_type& get_executor() const noexcept
     {
       return self_->get_executor();
     }
 
     template <typename SendHandler>
-    void operator()(ASIO_MOVE_ARG(SendHandler) handler,
-        ASIO_MOVE_ARG(payload_type) payload) const
+    void operator()(SendHandler&& handler,
+        payload_type&& payload) const
     {
       asio::detail::non_const_lvalue<SendHandler> handler2(handler);
       self_->service_->async_send(self_->impl_,
-          ASIO_MOVE_CAST(payload_type)(payload),
+          static_cast<payload_type&&>(payload),
           handler2.value, self_->get_executor());
     }
 
@@ -455,13 +477,13 @@ private:
     {
     }
 
-    const executor_type& get_executor() const ASIO_NOEXCEPT
+    const executor_type& get_executor() const noexcept
     {
       return self_->get_executor();
     }
 
     template <typename ReceiveHandler>
-    void operator()(ASIO_MOVE_ARG(ReceiveHandler) handler) const
+    void operator()(ReceiveHandler&& handler) const
     {
       asio::detail::non_const_lvalue<ReceiveHandler> handler2(handler);
       self_->service_->async_receive(self_->impl_,
