@@ -2,7 +2,7 @@
 // buffered_write_stream.cpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -17,6 +17,7 @@
 #include "asio/buffered_write_stream.hpp"
 
 #include <cstring>
+#include <functional>
 #include "archetypes/async_result.hpp"
 #include "asio/buffer.hpp"
 #include "asio/io_context.hpp"
@@ -30,12 +31,6 @@
 # include <array>
 #endif // defined(ASIO_HAS_BOOST_ARRAY)
 
-#if defined(ASIO_HAS_BOOST_BIND)
-# include <boost/bind/bind.hpp>
-#else // defined(ASIO_HAS_BOOST_BIND)
-# include <functional>
-#endif // defined(ASIO_HAS_BOOST_BIND)
-
 typedef asio::buffered_write_stream<
     asio::ip::tcp::socket> stream_type;
 
@@ -254,11 +249,7 @@ void test_async_operations()
 {
   using namespace std; // For memcmp.
 
-#if defined(ASIO_HAS_BOOST_BIND)
-  namespace bindns = boost;
-#else // defined(ASIO_HAS_BOOST_BIND)
   namespace bindns = std;
-#endif // defined(ASIO_HAS_BOOST_BIND)
   using bindns::placeholders::_1;
   using bindns::placeholders::_2;
 
