@@ -2,7 +2,7 @@
 // stream_protocol.cpp
 // ~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -215,5 +215,5 @@ void test()
 ASIO_TEST_SUITE
 (
   "local/stream_protocol",
-  ASIO_TEST_CASE(local_stream_protocol_socket_compile::test)
+  ASIO_COMPILE_TEST_CASE(local_stream_protocol_socket_compile::test)
 )
