@@ -2,7 +2,7 @@
 // stream_handle.cpp
 // ~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -67,20 +67,16 @@ void test()
     HANDLE native_handle2 = INVALID_HANDLE_VALUE;
     win::stream_handle handle4(ioc_ex, native_handle2);
 
-#if defined(ASIO_HAS_MOVE)
     win::stream_handle handle5(std::move(handle4));
 
     win::basic_stream_handle<io_context::executor_type> handle6(ioc);
     win::stream_handle handle7(std::move(handle6));
-#endif // defined(ASIO_HAS_MOVE)
 
     // basic_stream_handle operators.
 
-#if defined(ASIO_HAS_MOVE)
     handle1 = win::stream_handle(ioc);
     handle1 = std::move(handle4);
     handle1 = std::move(handle6);
-#endif // defined(ASIO_HAS_MOVE)
 
     // basic_io_object functions.
 
